package cn.iocoder.yudao.module.system.service.attendance.fixed;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.util.date.DateUtils;
import cn.iocoder.yudao.framework.common.util.distance.GeoUtil;
import cn.iocoder.yudao.framework.common.util.object.BeanUtils;
import cn.iocoder.yudao.module.system.controller.admin.fixed.vo.AttendanceFixedPageReqVO;
import cn.iocoder.yudao.module.system.controller.admin.fixed.vo.AttendanceFixedSaveReqVO;
import cn.iocoder.yudao.module.system.controller.app.attendance.dto.AttendancePunchPageDTO;
import cn.iocoder.yudao.module.system.controller.app.attendance.vo.AttendancePunchPageVO;
import cn.iocoder.yudao.module.system.dal.dataobject.attendance.fixed.AttendanceFixedDO;
import cn.iocoder.yudao.module.system.dal.dataobject.attendance.group.AttendanceGroupDO;
import cn.iocoder.yudao.module.system.dal.mysql.attendance.fixed.AttendanceFixedMapper;
import cn.iocoder.yudao.module.system.service.attendance.AttendanceService;
import cn.iocoder.yudao.module.system.service.attendance.punch.PunchService;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import org.springframework.context.annotation.Lazy;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;
import java.util.Date;
import java.util.List;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.system.enums.ErrorCodeConstants.FIXED_NOT_EXISTS;

/**
 * 固定班制考勤设置 Service 实现类
 *
 * @author 艾楷
 */
@Service("fixed")
@Validated
public class AttendanceFixedServiceImpl implements AttendanceFixedService, PunchService {
    @Resource
    private AttendanceFixedMapper fixedMapper;
    @Resource
    @Lazy // 避免依赖循环
    private AttendanceService attendanceService;
    @Resource
    private StringRedisTemplate stringRedisTemplate;

    @Override
    public Long createFixed(AttendanceFixedSaveReqVO createReqVO) {
        // 插入
        AttendanceFixedDO fixed = BeanUtils.toBean(createReqVO, AttendanceFixedDO.class);
        fixedMapper.insert(fixed);
        // 返回
        return fixed.getId();
    }

    @Override
    public void updateFixed(AttendanceFixedSaveReqVO updateReqVO) {
        // 校验存在
        validateFixedExists(updateReqVO.getId());
        // 更新
        AttendanceFixedDO updateObj = BeanUtils.toBean(updateReqVO, AttendanceFixedDO.class);
        fixedMapper.updateById(updateObj);
    }

    @Override
    public void deleteFixed(Long id) {
        // 校验存在
        validateFixedExists(id);
        // 删除
        fixedMapper.deleteById(id);
    }

    private void validateFixedExists(Long id) {
        if (fixedMapper.selectById(id) == null) {
            throw exception(FIXED_NOT_EXISTS);
        }
    }

    @Override
    public AttendanceFixedDO getFixed(Long id) {
        return fixedMapper.selectById(id);
    }

    @Override
    public PageResult<AttendanceFixedDO> getFixedPage(AttendanceFixedPageReqVO pageReqVO) {
        return fixedMapper.selectPage(pageReqVO);
    }

    @Override
    public AttendanceFixedDO getByGroupIdAndWeek(Long attendanceGroupId, Integer week) {
        List<AttendanceFixedDO> list = fixedMapper.selectList(new LambdaQueryWrapper<AttendanceFixedDO>()
                .eq(attendanceGroupId != null, AttendanceFixedDO::getAttendanceGroupId, attendanceGroupId)
                .eq(week != null, AttendanceFixedDO::getWeekTime, week));
        if (!list.isEmpty()) {
            return list.get(0);
        }
        return null;
    }

    @Override
    public AttendancePunchPageVO getPunchPage(AttendancePunchPageDTO dto) {
        AttendancePunchPageVO vo = new AttendancePunchPageVO();
        Date thisDate = new Date();
        AttendanceGroupDO activationGroup = dto.getActivationGroup();
        // -- 判断是否根据节假日自动排班 - 如果是的话 - 根据排班的来
        Boolean isHolidayFlag = AttendanceGroupDO.TRUE.equals(activationGroup.getAutoHolidaysFlag()) ?
                attendanceService.isHoliday(thisDate) : null;
        // -- 当前是节假日 并且是放假
        if (isHolidayFlag != null && isHolidayFlag) {
            return vo.setTodayNeedAttendance(AttendanceGroupDO.FALSE);
        }
        //获取到当天是周几
        int week = DateUtils.dayOfWeek(thisDate);
        AttendanceFixedDO attendanceFixedDO = this.getByGroupIdAndWeek(activationGroup.getId(), week);
        // -- 当前没有班次 - 不需要考勤
        if (attendanceFixedDO == null || attendanceFixedDO.getAttendanceGroupShiftId() == null) {
            return vo.setTodayNeedAttendance(AttendanceGroupDO.FALSE);
        }
        vo.setFieldworkFlag(activationGroup.getFieldworkFlag());
        // - 根据经纬度判断是否在对应班组的打卡点上
        vo.setCheckInPoint(GeoUtil.distance(Double.parseDouble(dto.getLatitude()), Double.parseDouble(dto.getLongitude())
                , Double.parseDouble(activationGroup.getLatitude()), Double.parseDouble(activationGroup.getLongitude()), activationGroup.getScope()));



        String key = "attendance" + "_" + activationGroup.getId();
        // -- 这里的key 有个情况 - 就是上班时间和下班时间夸天了 - 所以 这里要判断下上下班时间 -
        // 是否夸天 - 如果是的话 判断当前是上班卡还是下班卡 - 如果是下班卡的话 当前时间需要减去一天
        String mapKey = dto.getUserId() + "_" + DateUtils.dateFormat(thisDate, DateUtils.FORMAT_YEAR_MONTH_DAY);
        Object value = stringRedisTemplate.opsForHash().get(key, mapKey);
        if (value != null) {

        }

        return null;
    }


}